# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from .plans import (
    PlansResource,
    AsyncPlansResource,
    PlansResourceWithRawResponse,
    AsyncPlansResourceWithRawResponse,
    PlansResourceWithStreamingResponse,
    AsyncPlansResourceWithStreamingResponse,
)
from .alerts import (
    AlertsResource,
    AsyncAlertsResource,
    AlertsResourceWithRawResponse,
    AsyncAlertsResourceWithRawResponse,
    AlertsResourceWithStreamingResponse,
    AsyncAlertsResourceWithStreamingResponse,
)
from .commits import (
    CommitsResource,
    AsyncCommitsResource,
    CommitsResourceWithRawResponse,
    AsyncCommitsResourceWithRawResponse,
    CommitsResourceWithStreamingResponse,
    AsyncCommitsResourceWithStreamingResponse,
)
from .credits import (
    CreditsResource,
    AsyncCreditsResource,
    CreditsResourceWithRawResponse,
    AsyncCreditsResourceWithRawResponse,
    CreditsResourceWithStreamingResponse,
    AsyncCreditsResourceWithStreamingResponse,
)
from .invoices import (
    InvoicesResource,
    AsyncInvoicesResource,
    InvoicesResourceWithRawResponse,
    AsyncInvoicesResourceWithRawResponse,
    InvoicesResourceWithStreamingResponse,
    AsyncInvoicesResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ....types.v1 import (
    customer_list_params,
    customer_create_params,
    customer_archive_params,
    customer_set_name_params,
    customer_list_costs_params,
    customer_update_config_params,
    customer_preview_events_params,
    customer_set_ingest_aliases_params,
    customer_list_billable_metrics_params,
    customer_set_billing_configurations_params,
    customer_retrieve_billing_configurations_params,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursorPage, AsyncCursorPage
from .billing_config import (
    BillingConfigResource,
    AsyncBillingConfigResource,
    BillingConfigResourceWithRawResponse,
    AsyncBillingConfigResourceWithRawResponse,
    BillingConfigResourceWithStreamingResponse,
    AsyncBillingConfigResourceWithStreamingResponse,
)
from ...._base_client import AsyncPaginator, make_request_options
from .named_schedules import (
    NamedSchedulesResource,
    AsyncNamedSchedulesResource,
    NamedSchedulesResourceWithRawResponse,
    AsyncNamedSchedulesResourceWithRawResponse,
    NamedSchedulesResourceWithStreamingResponse,
    AsyncNamedSchedulesResourceWithStreamingResponse,
)
from ....types.v1.customer_detail import CustomerDetail
from ....types.v1.customer_create_response import CustomerCreateResponse
from ....types.v1.customer_archive_response import CustomerArchiveResponse
from ....types.v1.customer_retrieve_response import CustomerRetrieveResponse
from ....types.v1.customer_set_name_response import CustomerSetNameResponse
from ....types.v1.customer_list_costs_response import CustomerListCostsResponse
from ....types.v1.customer_preview_events_response import CustomerPreviewEventsResponse
from ....types.v1.customer_list_billable_metrics_response import CustomerListBillableMetricsResponse
from ....types.v1.customer_set_billing_configurations_response import CustomerSetBillingConfigurationsResponse
from ....types.v1.customer_retrieve_billing_configurations_response import CustomerRetrieveBillingConfigurationsResponse

__all__ = ["CustomersResource", "AsyncCustomersResource"]


class CustomersResource(SyncAPIResource):
    @cached_property
    def alerts(self) -> AlertsResource:
        return AlertsResource(self._client)

    @cached_property
    def plans(self) -> PlansResource:
        return PlansResource(self._client)

    @cached_property
    def invoices(self) -> InvoicesResource:
        return InvoicesResource(self._client)

    @cached_property
    def billing_config(self) -> BillingConfigResource:
        return BillingConfigResource(self._client)

    @cached_property
    def commits(self) -> CommitsResource:
        return CommitsResource(self._client)

    @cached_property
    def credits(self) -> CreditsResource:
        return CreditsResource(self._client)

    @cached_property
    def named_schedules(self) -> NamedSchedulesResource:
        return NamedSchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> CustomersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return CustomersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return CustomersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        billing_config: customer_create_params.BillingConfig | Omit = omit,
        custom_fields: Dict[str, str] | Omit = omit,
        customer_billing_provider_configurations: Iterable[customer_create_params.CustomerBillingProviderConfiguration]
        | Omit = omit,
        customer_revenue_system_configurations: Iterable[customer_create_params.CustomerRevenueSystemConfiguration]
        | Omit = omit,
        external_id: str | Omit = omit,
        ingest_aliases: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerCreateResponse:
        """
        Create a new customer in Metronome and optionally the billing configuration
        (recommended) which dictates where invoices for the customer will be sent or
        where payment will be collected.

        ### Use this endpoint to:

        Execute your customer provisioning workflows for either PLG motions, where
        customers originate in your platform, or SLG motions, where customers originate
        in your sales system.

        ### Key response fields:

        This end-point returns the `customer_id` created by the request. This id can be
        used to fetch relevant billing configurations and create contracts.

        ### Example workflow:

        - Generally, Metronome recommends first creating the customer in the downstream
          payment / ERP system when payment method is collected and then creating the
          customer in Metronome using the response (i.e. `customer_id`) from the
          downstream system. If you do not create a billing configuration on customer
          creation, you can add it later.
        - Once a customer is created, you can then create a contract for the customer.
          In the contract creation process, you will need to add the customer billing
          configuration to the contract to ensure Metronome invoices the customer
          correctly. This is because a customer can have multiple configurations.
        - As part of the customer creation process, set the ingest alias for the
          customer which will ensure usage is accurately mapped to the customer. Ingest
          aliases can be added or changed after the creation process as well.

        ### Usage guidelines:

        For details on different billing configurations for different systems, review
        the `/setCustomerBillingConfiguration` end-point.

        Args:
          name: This will be truncated to 160 characters if the provided name is longer.

          custom_fields: Custom fields to be added eg. { "key1": "value1", "key2": "value2" }

          external_id: (deprecated, use ingest_aliases instead) an alias that can be used to refer to
              this customer in usage events

          ingest_aliases: Aliases that can be used to refer to this customer in usage events

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/customers",
            body=maybe_transform(
                {
                    "name": name,
                    "billing_config": billing_config,
                    "custom_fields": custom_fields,
                    "customer_billing_provider_configurations": customer_billing_provider_configurations,
                    "customer_revenue_system_configurations": customer_revenue_system_configurations,
                    "external_id": external_id,
                    "ingest_aliases": ingest_aliases,
                },
                customer_create_params.CustomerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerCreateResponse,
        )

    def retrieve(
        self,
        *,
        customer_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerRetrieveResponse:
        """Get detailed information for a specific customer by their Metronome ID.

        Returns
        customer profile data including name, creation date, ingest aliases,
        configuration settings, and custom fields. Use this endpoint to fetch complete
        customer details for billing operations or account management.

        Note: If searching for a customer billing configuration, use the
        `/getCustomerBillingConfigurations` endpoint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._get(
            f"/v1/customers/{customer_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerRetrieveResponse,
        )

    def list(
        self,
        *,
        customer_ids: SequenceNotStr[str] | Omit = omit,
        ingest_alias: str | Omit = omit,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        only_archived: bool | Omit = omit,
        salesforce_account_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[CustomerDetail]:
        """Gets a paginated list of all customers in your Metronome account.

        Use this
        endpoint to browse your customer base, implement customer search functionality,
        or sync customer data with external systems. Returns customer details including
        IDs, names, and configuration settings. Supports filtering and pagination
        parameters for efficient data retrieval.

        Args:
          customer_ids: Filter the customer list by customer_id. Up to 100 ids can be provided.

          ingest_alias: Filter the customer list by ingest_alias

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          only_archived: Filter the customer list to only return archived customers. By default, only
              active customers are returned.

          salesforce_account_ids: Filter the customer list by salesforce_account_id. Up to 100 ids can be
              provided.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/customers",
            page=SyncCursorPage[CustomerDetail],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "customer_ids": customer_ids,
                        "ingest_alias": ingest_alias,
                        "limit": limit,
                        "next_page": next_page,
                        "only_archived": only_archived,
                        "salesforce_account_ids": salesforce_account_ids,
                    },
                    customer_list_params.CustomerListParams,
                ),
            ),
            model=CustomerDetail,
        )

    def archive(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerArchiveResponse:
        """Use this endpoint to archive a customer while preserving auditability.

        Archiving
        a customer will automatically archive all contracts as of the current date and
        void all corresponding invoices. Use this endpoint if a customer is onboarded by
        mistake.

        ### Usage guidelines:

        - Once a customer is archived, it cannot be unarchived.
        - Archived customers can still be viewed through the API or the UI for audit
          purposes.
        - Ingest aliases remain idempotent for archived customers. In order to reuse an
          ingest alias, first remove the ingest alias from the customer prior to
          archiving.
        - Any notifications associated with the customer will no longer be triggered.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/customers/archive",
            body=maybe_transform({"id": id}, customer_archive_params.CustomerArchiveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerArchiveResponse,
        )

    def list_billable_metrics(
        self,
        *,
        customer_id: str,
        include_archived: bool | Omit = omit,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        on_current_plan: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[CustomerListBillableMetricsResponse]:
        """Get all billable metrics available for a specific customer.

        Supports pagination
        and filtering by current plan status or archived metrics. Use this endpoint to
        see which metrics are being tracked for billing calculations for a given
        customer.

        Args:
          include_archived: If true, the list of returned metrics will include archived metrics

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          on_current_plan: If true, the list of metrics will be filtered to just ones that are on the
              customer's current plan

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._get_api_list(
            f"/v1/customers/{customer_id}/billable-metrics",
            page=SyncCursorPage[CustomerListBillableMetricsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_archived": include_archived,
                        "limit": limit,
                        "next_page": next_page,
                        "on_current_plan": on_current_plan,
                    },
                    customer_list_billable_metrics_params.CustomerListBillableMetricsParams,
                ),
            ),
            model=CustomerListBillableMetricsResponse,
        )

    def list_costs(
        self,
        *,
        customer_id: str,
        ending_before: Union[str, datetime],
        starting_on: Union[str, datetime],
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[CustomerListCostsResponse]:
        """
        Fetch daily pending costs for the specified customer, broken down by credit type
        and line items. Note: this is not supported for customers whose plan includes a
        UNIQUE-type billable metric. This is a Plans (deprecated) endpoint. New clients
        should implement using Contracts.

        Args:
          ending_before: RFC 3339 timestamp (exclusive)

          starting_on: RFC 3339 timestamp (inclusive)

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._get_api_list(
            f"/v1/customers/{customer_id}/costs",
            page=SyncCursorPage[CustomerListCostsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "starting_on": starting_on,
                        "limit": limit,
                        "next_page": next_page,
                    },
                    customer_list_costs_params.CustomerListCostsParams,
                ),
            ),
            model=CustomerListCostsResponse,
        )

    def preview_events(
        self,
        *,
        customer_id: str,
        events: Iterable[customer_preview_events_params.Event],
        mode: Literal["replace", "merge"] | Omit = omit,
        skip_zero_qty_line_items: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerPreviewEventsResponse:
        """Preview how a set of events will affect a customer's invoices.

        Generates draft
        invoices for a customer using their current contract configuration and the
        provided events. This is useful for testing how new events will affect the
        customer's invoices before they are actually processed. Customers on contracts
        with SQL billable metrics are not supported.

        Args:
          events: Array of usage events to include in the preview calculation. Must contain at
              least one event in `merge` mode.

          mode: Controls how the provided events are combined with existing usage data. Use
              `replace` to calculate the preview as if these are the only events for the
              customer, ignoring all historical usage. Use `merge` to combine these events
              with the customer's existing usage. Defaults to `replace`.

          skip_zero_qty_line_items: When `true`, line items with zero quantity are excluded from the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._post(
            f"/v1/customers/{customer_id}/previewEvents",
            body=maybe_transform(
                {
                    "events": events,
                    "mode": mode,
                    "skip_zero_qty_line_items": skip_zero_qty_line_items,
                },
                customer_preview_events_params.CustomerPreviewEventsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerPreviewEventsResponse,
        )

    def retrieve_billing_configurations(
        self,
        *,
        customer_id: str,
        include_archived: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerRetrieveBillingConfigurationsResponse:
        """Returns all billing configurations previously set for the customer.

        Use during
        the contract provisioning process to fetch the
        `billing_provider_configuration_id` needed to set the contract billing
        configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/getCustomerBillingProviderConfigurations",
            body=maybe_transform(
                {
                    "customer_id": customer_id,
                    "include_archived": include_archived,
                },
                customer_retrieve_billing_configurations_params.CustomerRetrieveBillingConfigurationsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerRetrieveBillingConfigurationsResponse,
        )

    def set_billing_configurations(
        self,
        *,
        data: Iterable[customer_set_billing_configurations_params.Data],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerSetBillingConfigurationsResponse:
        """Create a billing configuration for a customer.

        Once created, these
        configurations are available to associate to a contract and dictates which
        downstream system to collect payment in or send the invoice to. You can create
        multiple configurations per customer. The configuration formats are distinct for
        each downstream provider.

        ### Use this endpoint to:

        - Add the initial configuration to an existing customer. Once created, the
          billing configuration can then be associated to the customer's contract.
        - Add a new configuration to an existing customer. This might be used as part of
          an upgrade or downgrade workflow where the customer was previously billed
          through system A (e.g. Stripe) but will now be billed through system B (e.g.
          AWS). Once created, the new configuration can then be associated to the
          customer's contract.
        - Multiple configurations can be added per destination. For example, you can
          create two Stripe billing configurations for a Metronome customer that each
          have a distinct `collection_method`.

        ### Delivery method options:

        - `direct_to_billing_provider`: Use when Metronome should send invoices directly
          to the billing provider's API (e.g., Stripe, NetSuite). This is the most
          common method for automated billing workflows.
        - `tackle`: Use specifically for AWS Marketplace transactions that require
          Tackle's co-selling platform for partner attribution and commission tracking.
        - `aws_sqs`: Use when you want invoice data delivered to an AWS SQS queue for
          custom processing before sending to your billing system.
        - `aws_sns`: Use when you want invoice notifications published to an AWS SNS
          topic for event-driven billing workflows.

        ### Key response fields:

        The id for the customer billing configuration. This id can be used to associate
        the billing configuration to a contract.

        ### Usage guidelines:

        Must use the `delivery_method_id` if you have multiple Stripe accounts connected
        to Metronome.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/setCustomerBillingProviderConfigurations",
            body=maybe_transform(
                {"data": data}, customer_set_billing_configurations_params.CustomerSetBillingConfigurationsParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerSetBillingConfigurationsResponse,
        )

    def set_ingest_aliases(
        self,
        *,
        customer_id: str,
        ingest_aliases: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Sets the ingest aliases for a customer.

        Use this endpoint to associate a
        Metronome customer with an internal ID for easier tracking between systems.
        Ingest aliases can be used in the `customer_id` field when sending usage events
        to Metronome.

        ### Usage guidelines:

        - This call is idempotent and fully replaces the set of ingest aliases for the
          given customer.
        - Switching an ingest alias from one customer to another will associate all
          corresponding usage to the new customer.
        - Use multiple ingest aliases to model child organizations within a single
          Metronome customer.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/v1/customers/{customer_id}/setIngestAliases",
            body=maybe_transform(
                {"ingest_aliases": ingest_aliases}, customer_set_ingest_aliases_params.CustomerSetIngestAliasesParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def set_name(
        self,
        *,
        customer_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerSetNameResponse:
        """Updates the display name for a customer record.

        Use this to correct customer
        names, update business names after rebranding, or maintain accurate customer
        information for invoicing and reporting. Returns the updated customer object
        with the new name applied immediately across all billing documents and
        interfaces.

        Args:
          name: The new name for the customer. This will be truncated to 160 characters if the
              provided name is longer.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._post(
            f"/v1/customers/{customer_id}/setName",
            body=maybe_transform({"name": name}, customer_set_name_params.CustomerSetNameParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerSetNameResponse,
        )

    def update_config(
        self,
        *,
        customer_id: str,
        leave_stripe_invoices_in_draft: Optional[bool] | Omit = omit,
        salesforce_account_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Update configuration settings for a specific customer, such as external system
        integrations (e.g., Salesforce account ID) and other customer-specific billing
        parameters. Use this endpoint to modify customer configurations without
        affecting core customer data like name or ingest aliases.

        Args:
          leave_stripe_invoices_in_draft: Leave in draft or set to auto-advance on invoices sent to Stripe. Falls back to
              the client-level config if unset, which defaults to true if unset.

          salesforce_account_id: The Salesforce account ID for the customer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/v1/customers/{customer_id}/updateConfig",
            body=maybe_transform(
                {
                    "leave_stripe_invoices_in_draft": leave_stripe_invoices_in_draft,
                    "salesforce_account_id": salesforce_account_id,
                },
                customer_update_config_params.CustomerUpdateConfigParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncCustomersResource(AsyncAPIResource):
    @cached_property
    def alerts(self) -> AsyncAlertsResource:
        return AsyncAlertsResource(self._client)

    @cached_property
    def plans(self) -> AsyncPlansResource:
        return AsyncPlansResource(self._client)

    @cached_property
    def invoices(self) -> AsyncInvoicesResource:
        return AsyncInvoicesResource(self._client)

    @cached_property
    def billing_config(self) -> AsyncBillingConfigResource:
        return AsyncBillingConfigResource(self._client)

    @cached_property
    def commits(self) -> AsyncCommitsResource:
        return AsyncCommitsResource(self._client)

    @cached_property
    def credits(self) -> AsyncCreditsResource:
        return AsyncCreditsResource(self._client)

    @cached_property
    def named_schedules(self) -> AsyncNamedSchedulesResource:
        return AsyncNamedSchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCustomersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncCustomersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        billing_config: customer_create_params.BillingConfig | Omit = omit,
        custom_fields: Dict[str, str] | Omit = omit,
        customer_billing_provider_configurations: Iterable[customer_create_params.CustomerBillingProviderConfiguration]
        | Omit = omit,
        customer_revenue_system_configurations: Iterable[customer_create_params.CustomerRevenueSystemConfiguration]
        | Omit = omit,
        external_id: str | Omit = omit,
        ingest_aliases: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerCreateResponse:
        """
        Create a new customer in Metronome and optionally the billing configuration
        (recommended) which dictates where invoices for the customer will be sent or
        where payment will be collected.

        ### Use this endpoint to:

        Execute your customer provisioning workflows for either PLG motions, where
        customers originate in your platform, or SLG motions, where customers originate
        in your sales system.

        ### Key response fields:

        This end-point returns the `customer_id` created by the request. This id can be
        used to fetch relevant billing configurations and create contracts.

        ### Example workflow:

        - Generally, Metronome recommends first creating the customer in the downstream
          payment / ERP system when payment method is collected and then creating the
          customer in Metronome using the response (i.e. `customer_id`) from the
          downstream system. If you do not create a billing configuration on customer
          creation, you can add it later.
        - Once a customer is created, you can then create a contract for the customer.
          In the contract creation process, you will need to add the customer billing
          configuration to the contract to ensure Metronome invoices the customer
          correctly. This is because a customer can have multiple configurations.
        - As part of the customer creation process, set the ingest alias for the
          customer which will ensure usage is accurately mapped to the customer. Ingest
          aliases can be added or changed after the creation process as well.

        ### Usage guidelines:

        For details on different billing configurations for different systems, review
        the `/setCustomerBillingConfiguration` end-point.

        Args:
          name: This will be truncated to 160 characters if the provided name is longer.

          custom_fields: Custom fields to be added eg. { "key1": "value1", "key2": "value2" }

          external_id: (deprecated, use ingest_aliases instead) an alias that can be used to refer to
              this customer in usage events

          ingest_aliases: Aliases that can be used to refer to this customer in usage events

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/customers",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "billing_config": billing_config,
                    "custom_fields": custom_fields,
                    "customer_billing_provider_configurations": customer_billing_provider_configurations,
                    "customer_revenue_system_configurations": customer_revenue_system_configurations,
                    "external_id": external_id,
                    "ingest_aliases": ingest_aliases,
                },
                customer_create_params.CustomerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerCreateResponse,
        )

    async def retrieve(
        self,
        *,
        customer_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerRetrieveResponse:
        """Get detailed information for a specific customer by their Metronome ID.

        Returns
        customer profile data including name, creation date, ingest aliases,
        configuration settings, and custom fields. Use this endpoint to fetch complete
        customer details for billing operations or account management.

        Note: If searching for a customer billing configuration, use the
        `/getCustomerBillingConfigurations` endpoint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return await self._get(
            f"/v1/customers/{customer_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerRetrieveResponse,
        )

    def list(
        self,
        *,
        customer_ids: SequenceNotStr[str] | Omit = omit,
        ingest_alias: str | Omit = omit,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        only_archived: bool | Omit = omit,
        salesforce_account_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomerDetail, AsyncCursorPage[CustomerDetail]]:
        """Gets a paginated list of all customers in your Metronome account.

        Use this
        endpoint to browse your customer base, implement customer search functionality,
        or sync customer data with external systems. Returns customer details including
        IDs, names, and configuration settings. Supports filtering and pagination
        parameters for efficient data retrieval.

        Args:
          customer_ids: Filter the customer list by customer_id. Up to 100 ids can be provided.

          ingest_alias: Filter the customer list by ingest_alias

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          only_archived: Filter the customer list to only return archived customers. By default, only
              active customers are returned.

          salesforce_account_ids: Filter the customer list by salesforce_account_id. Up to 100 ids can be
              provided.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/customers",
            page=AsyncCursorPage[CustomerDetail],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "customer_ids": customer_ids,
                        "ingest_alias": ingest_alias,
                        "limit": limit,
                        "next_page": next_page,
                        "only_archived": only_archived,
                        "salesforce_account_ids": salesforce_account_ids,
                    },
                    customer_list_params.CustomerListParams,
                ),
            ),
            model=CustomerDetail,
        )

    async def archive(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerArchiveResponse:
        """Use this endpoint to archive a customer while preserving auditability.

        Archiving
        a customer will automatically archive all contracts as of the current date and
        void all corresponding invoices. Use this endpoint if a customer is onboarded by
        mistake.

        ### Usage guidelines:

        - Once a customer is archived, it cannot be unarchived.
        - Archived customers can still be viewed through the API or the UI for audit
          purposes.
        - Ingest aliases remain idempotent for archived customers. In order to reuse an
          ingest alias, first remove the ingest alias from the customer prior to
          archiving.
        - Any notifications associated with the customer will no longer be triggered.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/customers/archive",
            body=await async_maybe_transform({"id": id}, customer_archive_params.CustomerArchiveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerArchiveResponse,
        )

    def list_billable_metrics(
        self,
        *,
        customer_id: str,
        include_archived: bool | Omit = omit,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        on_current_plan: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomerListBillableMetricsResponse, AsyncCursorPage[CustomerListBillableMetricsResponse]]:
        """Get all billable metrics available for a specific customer.

        Supports pagination
        and filtering by current plan status or archived metrics. Use this endpoint to
        see which metrics are being tracked for billing calculations for a given
        customer.

        Args:
          include_archived: If true, the list of returned metrics will include archived metrics

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          on_current_plan: If true, the list of metrics will be filtered to just ones that are on the
              customer's current plan

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._get_api_list(
            f"/v1/customers/{customer_id}/billable-metrics",
            page=AsyncCursorPage[CustomerListBillableMetricsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_archived": include_archived,
                        "limit": limit,
                        "next_page": next_page,
                        "on_current_plan": on_current_plan,
                    },
                    customer_list_billable_metrics_params.CustomerListBillableMetricsParams,
                ),
            ),
            model=CustomerListBillableMetricsResponse,
        )

    def list_costs(
        self,
        *,
        customer_id: str,
        ending_before: Union[str, datetime],
        starting_on: Union[str, datetime],
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomerListCostsResponse, AsyncCursorPage[CustomerListCostsResponse]]:
        """
        Fetch daily pending costs for the specified customer, broken down by credit type
        and line items. Note: this is not supported for customers whose plan includes a
        UNIQUE-type billable metric. This is a Plans (deprecated) endpoint. New clients
        should implement using Contracts.

        Args:
          ending_before: RFC 3339 timestamp (exclusive)

          starting_on: RFC 3339 timestamp (inclusive)

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._get_api_list(
            f"/v1/customers/{customer_id}/costs",
            page=AsyncCursorPage[CustomerListCostsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "starting_on": starting_on,
                        "limit": limit,
                        "next_page": next_page,
                    },
                    customer_list_costs_params.CustomerListCostsParams,
                ),
            ),
            model=CustomerListCostsResponse,
        )

    async def preview_events(
        self,
        *,
        customer_id: str,
        events: Iterable[customer_preview_events_params.Event],
        mode: Literal["replace", "merge"] | Omit = omit,
        skip_zero_qty_line_items: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerPreviewEventsResponse:
        """Preview how a set of events will affect a customer's invoices.

        Generates draft
        invoices for a customer using their current contract configuration and the
        provided events. This is useful for testing how new events will affect the
        customer's invoices before they are actually processed. Customers on contracts
        with SQL billable metrics are not supported.

        Args:
          events: Array of usage events to include in the preview calculation. Must contain at
              least one event in `merge` mode.

          mode: Controls how the provided events are combined with existing usage data. Use
              `replace` to calculate the preview as if these are the only events for the
              customer, ignoring all historical usage. Use `merge` to combine these events
              with the customer's existing usage. Defaults to `replace`.

          skip_zero_qty_line_items: When `true`, line items with zero quantity are excluded from the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return await self._post(
            f"/v1/customers/{customer_id}/previewEvents",
            body=await async_maybe_transform(
                {
                    "events": events,
                    "mode": mode,
                    "skip_zero_qty_line_items": skip_zero_qty_line_items,
                },
                customer_preview_events_params.CustomerPreviewEventsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerPreviewEventsResponse,
        )

    async def retrieve_billing_configurations(
        self,
        *,
        customer_id: str,
        include_archived: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerRetrieveBillingConfigurationsResponse:
        """Returns all billing configurations previously set for the customer.

        Use during
        the contract provisioning process to fetch the
        `billing_provider_configuration_id` needed to set the contract billing
        configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/getCustomerBillingProviderConfigurations",
            body=await async_maybe_transform(
                {
                    "customer_id": customer_id,
                    "include_archived": include_archived,
                },
                customer_retrieve_billing_configurations_params.CustomerRetrieveBillingConfigurationsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerRetrieveBillingConfigurationsResponse,
        )

    async def set_billing_configurations(
        self,
        *,
        data: Iterable[customer_set_billing_configurations_params.Data],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerSetBillingConfigurationsResponse:
        """Create a billing configuration for a customer.

        Once created, these
        configurations are available to associate to a contract and dictates which
        downstream system to collect payment in or send the invoice to. You can create
        multiple configurations per customer. The configuration formats are distinct for
        each downstream provider.

        ### Use this endpoint to:

        - Add the initial configuration to an existing customer. Once created, the
          billing configuration can then be associated to the customer's contract.
        - Add a new configuration to an existing customer. This might be used as part of
          an upgrade or downgrade workflow where the customer was previously billed
          through system A (e.g. Stripe) but will now be billed through system B (e.g.
          AWS). Once created, the new configuration can then be associated to the
          customer's contract.
        - Multiple configurations can be added per destination. For example, you can
          create two Stripe billing configurations for a Metronome customer that each
          have a distinct `collection_method`.

        ### Delivery method options:

        - `direct_to_billing_provider`: Use when Metronome should send invoices directly
          to the billing provider's API (e.g., Stripe, NetSuite). This is the most
          common method for automated billing workflows.
        - `tackle`: Use specifically for AWS Marketplace transactions that require
          Tackle's co-selling platform for partner attribution and commission tracking.
        - `aws_sqs`: Use when you want invoice data delivered to an AWS SQS queue for
          custom processing before sending to your billing system.
        - `aws_sns`: Use when you want invoice notifications published to an AWS SNS
          topic for event-driven billing workflows.

        ### Key response fields:

        The id for the customer billing configuration. This id can be used to associate
        the billing configuration to a contract.

        ### Usage guidelines:

        Must use the `delivery_method_id` if you have multiple Stripe accounts connected
        to Metronome.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/setCustomerBillingProviderConfigurations",
            body=await async_maybe_transform(
                {"data": data}, customer_set_billing_configurations_params.CustomerSetBillingConfigurationsParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerSetBillingConfigurationsResponse,
        )

    async def set_ingest_aliases(
        self,
        *,
        customer_id: str,
        ingest_aliases: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Sets the ingest aliases for a customer.

        Use this endpoint to associate a
        Metronome customer with an internal ID for easier tracking between systems.
        Ingest aliases can be used in the `customer_id` field when sending usage events
        to Metronome.

        ### Usage guidelines:

        - This call is idempotent and fully replaces the set of ingest aliases for the
          given customer.
        - Switching an ingest alias from one customer to another will associate all
          corresponding usage to the new customer.
        - Use multiple ingest aliases to model child organizations within a single
          Metronome customer.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/v1/customers/{customer_id}/setIngestAliases",
            body=await async_maybe_transform(
                {"ingest_aliases": ingest_aliases}, customer_set_ingest_aliases_params.CustomerSetIngestAliasesParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def set_name(
        self,
        *,
        customer_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerSetNameResponse:
        """Updates the display name for a customer record.

        Use this to correct customer
        names, update business names after rebranding, or maintain accurate customer
        information for invoicing and reporting. Returns the updated customer object
        with the new name applied immediately across all billing documents and
        interfaces.

        Args:
          name: The new name for the customer. This will be truncated to 160 characters if the
              provided name is longer.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return await self._post(
            f"/v1/customers/{customer_id}/setName",
            body=await async_maybe_transform({"name": name}, customer_set_name_params.CustomerSetNameParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerSetNameResponse,
        )

    async def update_config(
        self,
        *,
        customer_id: str,
        leave_stripe_invoices_in_draft: Optional[bool] | Omit = omit,
        salesforce_account_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Update configuration settings for a specific customer, such as external system
        integrations (e.g., Salesforce account ID) and other customer-specific billing
        parameters. Use this endpoint to modify customer configurations without
        affecting core customer data like name or ingest aliases.

        Args:
          leave_stripe_invoices_in_draft: Leave in draft or set to auto-advance on invoices sent to Stripe. Falls back to
              the client-level config if unset, which defaults to true if unset.

          salesforce_account_id: The Salesforce account ID for the customer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/v1/customers/{customer_id}/updateConfig",
            body=await async_maybe_transform(
                {
                    "leave_stripe_invoices_in_draft": leave_stripe_invoices_in_draft,
                    "salesforce_account_id": salesforce_account_id,
                },
                customer_update_config_params.CustomerUpdateConfigParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class CustomersResourceWithRawResponse:
    def __init__(self, customers: CustomersResource) -> None:
        self._customers = customers

        self.create = to_raw_response_wrapper(
            customers.create,
        )
        self.retrieve = to_raw_response_wrapper(
            customers.retrieve,
        )
        self.list = to_raw_response_wrapper(
            customers.list,
        )
        self.archive = to_raw_response_wrapper(
            customers.archive,
        )
        self.list_billable_metrics = to_raw_response_wrapper(
            customers.list_billable_metrics,
        )
        self.list_costs = to_raw_response_wrapper(
            customers.list_costs,
        )
        self.preview_events = to_raw_response_wrapper(
            customers.preview_events,
        )
        self.retrieve_billing_configurations = to_raw_response_wrapper(
            customers.retrieve_billing_configurations,
        )
        self.set_billing_configurations = to_raw_response_wrapper(
            customers.set_billing_configurations,
        )
        self.set_ingest_aliases = to_raw_response_wrapper(
            customers.set_ingest_aliases,
        )
        self.set_name = to_raw_response_wrapper(
            customers.set_name,
        )
        self.update_config = to_raw_response_wrapper(
            customers.update_config,
        )

    @cached_property
    def alerts(self) -> AlertsResourceWithRawResponse:
        return AlertsResourceWithRawResponse(self._customers.alerts)

    @cached_property
    def plans(self) -> PlansResourceWithRawResponse:
        return PlansResourceWithRawResponse(self._customers.plans)

    @cached_property
    def invoices(self) -> InvoicesResourceWithRawResponse:
        return InvoicesResourceWithRawResponse(self._customers.invoices)

    @cached_property
    def billing_config(self) -> BillingConfigResourceWithRawResponse:
        return BillingConfigResourceWithRawResponse(self._customers.billing_config)

    @cached_property
    def commits(self) -> CommitsResourceWithRawResponse:
        return CommitsResourceWithRawResponse(self._customers.commits)

    @cached_property
    def credits(self) -> CreditsResourceWithRawResponse:
        return CreditsResourceWithRawResponse(self._customers.credits)

    @cached_property
    def named_schedules(self) -> NamedSchedulesResourceWithRawResponse:
        return NamedSchedulesResourceWithRawResponse(self._customers.named_schedules)


class AsyncCustomersResourceWithRawResponse:
    def __init__(self, customers: AsyncCustomersResource) -> None:
        self._customers = customers

        self.create = async_to_raw_response_wrapper(
            customers.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            customers.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            customers.list,
        )
        self.archive = async_to_raw_response_wrapper(
            customers.archive,
        )
        self.list_billable_metrics = async_to_raw_response_wrapper(
            customers.list_billable_metrics,
        )
        self.list_costs = async_to_raw_response_wrapper(
            customers.list_costs,
        )
        self.preview_events = async_to_raw_response_wrapper(
            customers.preview_events,
        )
        self.retrieve_billing_configurations = async_to_raw_response_wrapper(
            customers.retrieve_billing_configurations,
        )
        self.set_billing_configurations = async_to_raw_response_wrapper(
            customers.set_billing_configurations,
        )
        self.set_ingest_aliases = async_to_raw_response_wrapper(
            customers.set_ingest_aliases,
        )
        self.set_name = async_to_raw_response_wrapper(
            customers.set_name,
        )
        self.update_config = async_to_raw_response_wrapper(
            customers.update_config,
        )

    @cached_property
    def alerts(self) -> AsyncAlertsResourceWithRawResponse:
        return AsyncAlertsResourceWithRawResponse(self._customers.alerts)

    @cached_property
    def plans(self) -> AsyncPlansResourceWithRawResponse:
        return AsyncPlansResourceWithRawResponse(self._customers.plans)

    @cached_property
    def invoices(self) -> AsyncInvoicesResourceWithRawResponse:
        return AsyncInvoicesResourceWithRawResponse(self._customers.invoices)

    @cached_property
    def billing_config(self) -> AsyncBillingConfigResourceWithRawResponse:
        return AsyncBillingConfigResourceWithRawResponse(self._customers.billing_config)

    @cached_property
    def commits(self) -> AsyncCommitsResourceWithRawResponse:
        return AsyncCommitsResourceWithRawResponse(self._customers.commits)

    @cached_property
    def credits(self) -> AsyncCreditsResourceWithRawResponse:
        return AsyncCreditsResourceWithRawResponse(self._customers.credits)

    @cached_property
    def named_schedules(self) -> AsyncNamedSchedulesResourceWithRawResponse:
        return AsyncNamedSchedulesResourceWithRawResponse(self._customers.named_schedules)


class CustomersResourceWithStreamingResponse:
    def __init__(self, customers: CustomersResource) -> None:
        self._customers = customers

        self.create = to_streamed_response_wrapper(
            customers.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            customers.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            customers.list,
        )
        self.archive = to_streamed_response_wrapper(
            customers.archive,
        )
        self.list_billable_metrics = to_streamed_response_wrapper(
            customers.list_billable_metrics,
        )
        self.list_costs = to_streamed_response_wrapper(
            customers.list_costs,
        )
        self.preview_events = to_streamed_response_wrapper(
            customers.preview_events,
        )
        self.retrieve_billing_configurations = to_streamed_response_wrapper(
            customers.retrieve_billing_configurations,
        )
        self.set_billing_configurations = to_streamed_response_wrapper(
            customers.set_billing_configurations,
        )
        self.set_ingest_aliases = to_streamed_response_wrapper(
            customers.set_ingest_aliases,
        )
        self.set_name = to_streamed_response_wrapper(
            customers.set_name,
        )
        self.update_config = to_streamed_response_wrapper(
            customers.update_config,
        )

    @cached_property
    def alerts(self) -> AlertsResourceWithStreamingResponse:
        return AlertsResourceWithStreamingResponse(self._customers.alerts)

    @cached_property
    def plans(self) -> PlansResourceWithStreamingResponse:
        return PlansResourceWithStreamingResponse(self._customers.plans)

    @cached_property
    def invoices(self) -> InvoicesResourceWithStreamingResponse:
        return InvoicesResourceWithStreamingResponse(self._customers.invoices)

    @cached_property
    def billing_config(self) -> BillingConfigResourceWithStreamingResponse:
        return BillingConfigResourceWithStreamingResponse(self._customers.billing_config)

    @cached_property
    def commits(self) -> CommitsResourceWithStreamingResponse:
        return CommitsResourceWithStreamingResponse(self._customers.commits)

    @cached_property
    def credits(self) -> CreditsResourceWithStreamingResponse:
        return CreditsResourceWithStreamingResponse(self._customers.credits)

    @cached_property
    def named_schedules(self) -> NamedSchedulesResourceWithStreamingResponse:
        return NamedSchedulesResourceWithStreamingResponse(self._customers.named_schedules)


class AsyncCustomersResourceWithStreamingResponse:
    def __init__(self, customers: AsyncCustomersResource) -> None:
        self._customers = customers

        self.create = async_to_streamed_response_wrapper(
            customers.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            customers.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            customers.list,
        )
        self.archive = async_to_streamed_response_wrapper(
            customers.archive,
        )
        self.list_billable_metrics = async_to_streamed_response_wrapper(
            customers.list_billable_metrics,
        )
        self.list_costs = async_to_streamed_response_wrapper(
            customers.list_costs,
        )
        self.preview_events = async_to_streamed_response_wrapper(
            customers.preview_events,
        )
        self.retrieve_billing_configurations = async_to_streamed_response_wrapper(
            customers.retrieve_billing_configurations,
        )
        self.set_billing_configurations = async_to_streamed_response_wrapper(
            customers.set_billing_configurations,
        )
        self.set_ingest_aliases = async_to_streamed_response_wrapper(
            customers.set_ingest_aliases,
        )
        self.set_name = async_to_streamed_response_wrapper(
            customers.set_name,
        )
        self.update_config = async_to_streamed_response_wrapper(
            customers.update_config,
        )

    @cached_property
    def alerts(self) -> AsyncAlertsResourceWithStreamingResponse:
        return AsyncAlertsResourceWithStreamingResponse(self._customers.alerts)

    @cached_property
    def plans(self) -> AsyncPlansResourceWithStreamingResponse:
        return AsyncPlansResourceWithStreamingResponse(self._customers.plans)

    @cached_property
    def invoices(self) -> AsyncInvoicesResourceWithStreamingResponse:
        return AsyncInvoicesResourceWithStreamingResponse(self._customers.invoices)

    @cached_property
    def billing_config(self) -> AsyncBillingConfigResourceWithStreamingResponse:
        return AsyncBillingConfigResourceWithStreamingResponse(self._customers.billing_config)

    @cached_property
    def commits(self) -> AsyncCommitsResourceWithStreamingResponse:
        return AsyncCommitsResourceWithStreamingResponse(self._customers.commits)

    @cached_property
    def credits(self) -> AsyncCreditsResourceWithStreamingResponse:
        return AsyncCreditsResourceWithStreamingResponse(self._customers.credits)

    @cached_property
    def named_schedules(self) -> AsyncNamedSchedulesResourceWithStreamingResponse:
        return AsyncNamedSchedulesResourceWithStreamingResponse(self._customers.named_schedules)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .commit import Commit
from .credit import Credit
from .discount import Discount
from .override import Override
from ..._models import BaseModel
from .pro_service import ProService
from .commit_specifier import CommitSpecifier
from .scheduled_charge import ScheduledCharge
from .base_usage_filter import BaseUsageFilter
from .hierarchy_configuration import HierarchyConfiguration
from .spend_threshold_configuration import SpendThresholdConfiguration
from .commit_hierarchy_configuration import CommitHierarchyConfiguration
from .recurring_commit_subscription_config import RecurringCommitSubscriptionConfig
from .prepaid_balance_threshold_configuration import PrepaidBalanceThresholdConfiguration

__all__ = [
    "ContractWithoutAmendments",
    "Transition",
    "UsageStatementSchedule",
    "RecurringCommit",
    "RecurringCommitAccessAmount",
    "RecurringCommitCommitDuration",
    "RecurringCommitProduct",
    "RecurringCommitContract",
    "RecurringCommitInvoiceAmount",
    "RecurringCredit",
    "RecurringCreditAccessAmount",
    "RecurringCreditCommitDuration",
    "RecurringCreditProduct",
    "RecurringCreditContract",
    "ResellerRoyalty",
    "UsageFilter",
    "UsageFilterUpdate",
]


class Transition(BaseModel):
    from_contract_id: str

    to_contract_id: str

    type: Literal["SUPERSEDE", "RENEWAL"]


class UsageStatementSchedule(BaseModel):
    billing_anchor_date: datetime
    """Contract usage statements follow a selected cadence based on this date."""

    frequency: Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]


class RecurringCommitAccessAmount(BaseModel):
    """The amount of commit to grant."""

    credit_type_id: str

    unit_price: float

    quantity: Optional[float] = None


class RecurringCommitCommitDuration(BaseModel):
    """The amount of time the created commits will be valid for"""

    value: float

    unit: Optional[Literal["PERIODS"]] = None


class RecurringCommitProduct(BaseModel):
    id: str

    name: str


class RecurringCommitContract(BaseModel):
    id: str


class RecurringCommitInvoiceAmount(BaseModel):
    """The amount the customer should be billed for the commit. Not required."""

    credit_type_id: str

    quantity: float

    unit_price: float


class RecurringCommit(BaseModel):
    id: str

    access_amount: RecurringCommitAccessAmount
    """The amount of commit to grant."""

    commit_duration: RecurringCommitCommitDuration
    """The amount of time the created commits will be valid for"""

    priority: float
    """Will be passed down to the individual commits"""

    product: RecurringCommitProduct

    rate_type: Literal["COMMIT_RATE", "LIST_RATE"]
    """Whether the created commits will use the commit rate or list rate"""

    starting_at: datetime
    """Determines the start time for the first commit"""

    applicable_product_ids: Optional[List[str]] = None
    """Will be passed down to the individual commits"""

    applicable_product_tags: Optional[List[str]] = None
    """Will be passed down to the individual commits"""

    contract: Optional[RecurringCommitContract] = None

    description: Optional[str] = None
    """Will be passed down to the individual commits"""

    ending_before: Optional[datetime] = None
    """Determines when the contract will stop creating recurring commits. Optional"""

    hierarchy_configuration: Optional[CommitHierarchyConfiguration] = None
    """Optional configuration for recurring commit/credit hierarchy access control"""

    invoice_amount: Optional[RecurringCommitInvoiceAmount] = None
    """The amount the customer should be billed for the commit. Not required."""

    name: Optional[str] = None
    """Displayed on invoices. Will be passed through to the individual commits"""

    netsuite_sales_order_id: Optional[str] = None
    """Will be passed down to the individual commits"""

    proration: Optional[Literal["NONE", "FIRST", "LAST", "FIRST_AND_LAST"]] = None
    """Determines whether the first and last commit will be prorated.

    If not provided, the default is FIRST_AND_LAST (i.e. prorate both the first and
    last commits).
    """

    recurrence_frequency: Optional[Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]] = None
    """The frequency at which the recurring commits will be created.

    If not provided: - The commits will be created on the usage invoice frequency.
    If provided: - The period defined in the duration will correspond to this
    frequency. - Commits will be created aligned with the recurring commit's
    starting_at rather than the usage invoice dates.
    """

    rollover_fraction: Optional[float] = None
    """Will be passed down to the individual commits.

    This controls how much of an individual unexpired commit will roll over upon
    contract transition. Must be between 0 and 1.
    """

    specifiers: Optional[List[CommitSpecifier]] = None
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown.
    """

    subscription_config: Optional[RecurringCommitSubscriptionConfig] = None
    """Attach a subscription to the recurring commit/credit."""


class RecurringCreditAccessAmount(BaseModel):
    """The amount of commit to grant."""

    credit_type_id: str

    unit_price: float

    quantity: Optional[float] = None


class RecurringCreditCommitDuration(BaseModel):
    """The amount of time the created commits will be valid for"""

    value: float

    unit: Optional[Literal["PERIODS"]] = None


class RecurringCreditProduct(BaseModel):
    id: str

    name: str


class RecurringCreditContract(BaseModel):
    id: str


class RecurringCredit(BaseModel):
    id: str

    access_amount: RecurringCreditAccessAmount
    """The amount of commit to grant."""

    commit_duration: RecurringCreditCommitDuration
    """The amount of time the created commits will be valid for"""

    priority: float
    """Will be passed down to the individual commits"""

    product: RecurringCreditProduct

    rate_type: Literal["COMMIT_RATE", "LIST_RATE"]
    """Whether the created commits will use the commit rate or list rate"""

    starting_at: datetime
    """Determines the start time for the first commit"""

    applicable_product_ids: Optional[List[str]] = None
    """Will be passed down to the individual commits"""

    applicable_product_tags: Optional[List[str]] = None
    """Will be passed down to the individual commits"""

    contract: Optional[RecurringCreditContract] = None

    description: Optional[str] = None
    """Will be passed down to the individual commits"""

    ending_before: Optional[datetime] = None
    """Determines when the contract will stop creating recurring commits. Optional"""

    hierarchy_configuration: Optional[CommitHierarchyConfiguration] = None
    """Optional configuration for recurring commit/credit hierarchy access control"""

    name: Optional[str] = None
    """Displayed on invoices. Will be passed through to the individual commits"""

    netsuite_sales_order_id: Optional[str] = None
    """Will be passed down to the individual commits"""

    proration: Optional[Literal["NONE", "FIRST", "LAST", "FIRST_AND_LAST"]] = None
    """Determines whether the first and last commit will be prorated.

    If not provided, the default is FIRST_AND_LAST (i.e. prorate both the first and
    last commits).
    """

    recurrence_frequency: Optional[Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]] = None
    """The frequency at which the recurring commits will be created.

    If not provided: - The commits will be created on the usage invoice frequency.
    If provided: - The period defined in the duration will correspond to this
    frequency. - Commits will be created aligned with the recurring commit's
    starting_at rather than the usage invoice dates.
    """

    rollover_fraction: Optional[float] = None
    """Will be passed down to the individual commits.

    This controls how much of an individual unexpired commit will roll over upon
    contract transition. Must be between 0 and 1.
    """

    specifiers: Optional[List[CommitSpecifier]] = None
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown.
    """

    subscription_config: Optional[RecurringCommitSubscriptionConfig] = None
    """Attach a subscription to the recurring commit/credit."""


class ResellerRoyalty(BaseModel):
    fraction: float

    netsuite_reseller_id: str

    reseller_type: Literal["AWS", "AWS_PRO_SERVICE", "GCP", "GCP_PRO_SERVICE"]

    starting_at: datetime

    applicable_product_ids: Optional[List[str]] = None

    applicable_product_tags: Optional[List[str]] = None

    aws_account_number: Optional[str] = None

    aws_offer_id: Optional[str] = None

    aws_payer_reference_id: Optional[str] = None

    ending_before: Optional[datetime] = None

    gcp_account_id: Optional[str] = None

    gcp_offer_id: Optional[str] = None

    reseller_contract_value: Optional[float] = None


class UsageFilterUpdate(BaseModel):
    group_key: str

    group_values: List[str]

    starting_at: datetime


class UsageFilter(BaseModel):
    current: Optional[BaseUsageFilter] = None

    initial: BaseUsageFilter

    updates: List[UsageFilterUpdate]


class ContractWithoutAmendments(BaseModel):
    commits: List[Commit]

    created_at: datetime

    created_by: str

    overrides: List[Override]

    scheduled_charges: List[ScheduledCharge]

    starting_at: datetime

    transitions: List[Transition]

    usage_statement_schedule: UsageStatementSchedule

    credits: Optional[List[Credit]] = None

    discounts: Optional[List[Discount]] = None
    """This field's availability is dependent on your client's"""

    ending_before: Optional[datetime] = None

    hierarchy_configuration: Optional[HierarchyConfiguration] = None
    """
    Either a **parent** configuration with a list of children or a **child**
    configuration with a single parent.
    """

    name: Optional[str] = None

    net_payment_terms_days: Optional[float] = None

    netsuite_sales_order_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    prepaid_balance_threshold_configuration: Optional[PrepaidBalanceThresholdConfiguration] = None

    professional_services: Optional[List[ProService]] = None
    """This field's availability is dependent on your client's configuration."""

    rate_card_id: Optional[str] = None

    recurring_commits: Optional[List[RecurringCommit]] = None

    recurring_credits: Optional[List[RecurringCredit]] = None

    reseller_royalties: Optional[List[ResellerRoyalty]] = None
    """This field's availability is dependent on your client's configuration."""

    salesforce_opportunity_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    scheduled_charges_on_usage_invoices: Optional[Literal["ALL"]] = None
    """
    Determines which scheduled and commit charges to consolidate onto the Contract's
    usage invoice. The charge's `timestamp` must match the usage invoice's
    `ending_before` date for consolidation to occur. This field cannot be modified
    after a Contract has been created. If this field is omitted, charges will appear
    on a separate invoice from usage charges.
    """

    spend_threshold_configuration: Optional[SpendThresholdConfiguration] = None

    total_contract_value: Optional[float] = None
    """This field's availability is dependent on your client's configuration."""

    usage_filter: Optional[UsageFilter] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["BaseThresholdCommit"]


class BaseThresholdCommit(TypedDict, total=False):
    product_id: Required[str]
    """
    The commit product that will be used to generate the line item for commit
    payment.
    """

    description: str

    name: str
    """Specify the name of the line item for the threshold charge.

    If left blank, it will default to the commit product name.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from ..._models import BaseModel
from .customer_detail import CustomerDetail

__all__ = ["PlanListCustomersResponse", "PlanDetails"]


class PlanDetails(BaseModel):
    id: str

    custom_fields: Dict[str, str]
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    customer_plan_id: str

    name: str

    starting_on: datetime
    """The start date of the plan"""

    ending_before: Optional[datetime] = None
    """The end date of the plan"""


class PlanListCustomersResponse(BaseModel):
    customer_details: CustomerDetail

    plan_details: PlanDetails



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from ..shared.credit_type_data import CreditTypeData

__all__ = ["PlanListChargesResponse", "Price", "UnitConversion"]


class Price(BaseModel):
    tier: float
    """Used in pricing tiers. Indicates at what metric value the price applies."""

    value: float

    collection_interval: Optional[float] = None

    collection_schedule: Optional[str] = None

    quantity: Optional[float] = None


class UnitConversion(BaseModel):
    """Specifies how quantities for usage based charges will be converted."""

    division_factor: float
    """The conversion factor"""

    rounding_behavior: Optional[Literal["floor", "ceiling"]] = None
    """Whether usage should be rounded down or up to the nearest whole number.

    If null, quantity will be rounded to 20 decimal places.
    """


class PlanListChargesResponse(BaseModel):
    id: str

    charge_type: Literal["usage", "fixed", "composite", "minimum", "seat"]

    credit_type: CreditTypeData

    custom_fields: Dict[str, str]
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    name: str

    prices: List[Price]

    product_id: str

    product_name: str

    quantity: Optional[float] = None

    start_period: Optional[float] = None
    """Used in price ramps.

    Indicates how many billing periods pass before the charge applies.
    """

    tier_reset_frequency: Optional[float] = None
    """Used in pricing tiers.

    Indicates how often the tier resets. Default is 1 - the tier count resets every
    billing period.
    """

    unit_conversion: Optional[UnitConversion] = None
    """Specifies how quantities for usage based charges will be converted."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

__all__ = ["BillingProviderCreateParams"]


class BillingProviderCreateParams(TypedDict, total=False):
    billing_provider: Required[Literal["aws_marketplace", "azure_marketplace", "gcp_marketplace"]]
    """The billing provider set for this configuration."""

    configuration: Required[Dict[str, object]]
    """Account-level configuration for the billing provider.

    The structure of this object is specific to the billing provider and delivery
    provider combination. See examples below.
    """

    delivery_method: Required[Literal["direct_to_billing_provider", "aws_sqs", "aws_sns"]]
    """The method to use for delivering invoices for this configuration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["PaymentStatus"]

PaymentStatus: TypeAlias = Literal["pending", "requires_intervention", "paid", "canceled"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from metronome import Metronome, AsyncMetronome
from tests.utils import assert_matches_type
from metronome._utils import parse_datetime
from metronome.pagination import SyncCursorPage, AsyncCursorPage
from metronome.types.v1.contracts import (
    ProductListResponse,
    ProductCreateResponse,
    ProductUpdateResponse,
    ProductArchiveResponse,
    ProductRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestProducts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Metronome) -> None:
        product = client.v1.contracts.products.create(
            name="My Product",
            type="USAGE",
        )
        assert_matches_type(ProductCreateResponse, product, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Metronome) -> None:
        product = client.v1.contracts.products.create(
            name="My Product",
            type="USAGE",
            billable_metric_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
            composite_product_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            composite_tags=["string"],
            custom_fields={"foo": "string"},
            exclude_free_usage=True,
            is_refundable=True,
            netsuite_internal_item_id="netsuite_internal_item_id",
            netsuite_overage_item_id="netsuite_overage_item_id",
            presentation_group_key=["string"],
            pricing_group_key=["string"],
            quantity_conversion={
                "conversion_factor": 0,
                "operation": "MULTIPLY",
                "name": "name",
            },
            quantity_rounding={
                "decimal_places": 0,
                "rounding_method": "ROUND_UP",
            },
            tags=["string"],
        )
        assert_matches_type(ProductCreateResponse, product, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Metronome) -> None:
        response = client.v1.contracts.products.with_raw_response.create(
            name="My Product",
            type="USAGE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(ProductCreateResponse, product, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Metronome) -> None:
        with client.v1.contracts.products.with_streaming_response.create(
            name="My Product",
            type="USAGE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(ProductCreateResponse, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Metronome) -> None:
        product = client.v1.contracts.products.retrieve(
            id="d84e7f4e-7a70-4fe4-be02-7a5027beffcc",
        )
        assert_matches_type(ProductRetrieveResponse, product, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Metronome) -> None:
        response = client.v1.contracts.products.with_raw_response.retrieve(
            id="d84e7f4e-7a70-4fe4-be02-7a5027beffcc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(ProductRetrieveResponse, product, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Metronome) -> None:
        with client.v1.contracts.products.with_streaming_response.retrieve(
            id="d84e7f4e-7a70-4fe4-be02-7a5027beffcc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(ProductRetrieveResponse, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_update(self, client: Metronome) -> None:
        product = client.v1.contracts.products.update(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            starting_at=parse_datetime("2020-01-01T00:00:00.000Z"),
        )
        assert_matches_type(ProductUpdateResponse, product, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Metronome) -> None:
        product = client.v1.contracts.products.update(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            starting_at=parse_datetime("2020-01-01T00:00:00.000Z"),
            billable_metric_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            composite_product_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            composite_tags=["string"],
            exclude_free_usage=True,
            is_refundable=True,
            name="My Updated Product",
            netsuite_internal_item_id="netsuite_internal_item_id",
            netsuite_overage_item_id="netsuite_overage_item_id",
            presentation_group_key=["string"],
            pricing_group_key=["string"],
            quantity_conversion={
                "conversion_factor": 0,
                "operation": "MULTIPLY",
                "name": "name",
            },
            quantity_rounding={
                "decimal_places": 0,
                "rounding_method": "ROUND_UP",
            },
            tags=["string"],
        )
        assert_matches_type(ProductUpdateResponse, product, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Metronome) -> None:
        response = client.v1.contracts.products.with_raw_response.update(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            starting_at=parse_datetime("2020-01-01T00:00:00.000Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(ProductUpdateResponse, product, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Metronome) -> None:
        with client.v1.contracts.products.with_streaming_response.update(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            starting_at=parse_datetime("2020-01-01T00:00:00.000Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(ProductUpdateResponse, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list(self, client: Metronome) -> None:
        product = client.v1.contracts.products.list()
        assert_matches_type(SyncCursorPage[ProductListResponse], product, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Metronome) -> None:
        product = client.v1.contracts.products.list(
            limit=1,
            next_page="next_page",
            archive_filter="NOT_ARCHIVED",
        )
        assert_matches_type(SyncCursorPage[ProductListResponse], product, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Metronome) -> None:
        response = client.v1.contracts.products.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(SyncCursorPage[ProductListResponse], product, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Metronome) -> None:
        with client.v1.contracts.products.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(SyncCursorPage[ProductListResponse], product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_archive(self, client: Metronome) -> None:
        product = client.v1.contracts.products.archive(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(ProductArchiveResponse, product, path=["response"])

    @parametrize
    def test_raw_response_archive(self, client: Metronome) -> None:
        response = client.v1.contracts.products.with_raw_response.archive(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(ProductArchiveResponse, product, path=["response"])

    @parametrize
    def test_streaming_response_archive(self, client: Metronome) -> None:
        with client.v1.contracts.products.with_streaming_response.archive(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(ProductArchiveResponse, product, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncProducts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncMetronome) -> None:
        product = await async_client.v1.contracts.products.create(
            name="My Product",
            type="USAGE",
        )
        assert_matches_type(ProductCreateResponse, product, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncMetronome) -> None:
        product = await async_client.v1.contracts.products.create(
            name="My Product",
            type="USAGE",
            billable_metric_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
            composite_product_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            composite_tags=["string"],
            custom_fields={"foo": "string"},
            exclude_free_usage=True,
            is_refundable=True,
            netsuite_internal_item_id="netsuite_internal_item_id",
            netsuite_overage_item_id="netsuite_overage_item_id",
            presentation_group_key=["string"],
            pricing_group_key=["string"],
            quantity_conversion={
                "conversion_factor": 0,
                "operation": "MULTIPLY",
                "name": "name",
            },
            quantity_rounding={
                "decimal_places": 0,
                "rounding_method": "ROUND_UP",
            },
            tags=["string"],
        )
        assert_matches_type(ProductCreateResponse, product, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.contracts.products.with_raw_response.create(
            name="My Product",
            type="USAGE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(ProductCreateResponse, product, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.contracts.products.with_streaming_response.create(
            name="My Product",
            type="USAGE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(ProductCreateResponse, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncMetronome) -> None:
        product = await async_client.v1.contracts.products.retrieve(
            id="d84e7f4e-7a70-4fe4-be02-7a5027beffcc",
        )
        assert_matches_type(ProductRetrieveResponse, product, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.contracts.products.with_raw_response.retrieve(
            id="d84e7f4e-7a70-4fe4-be02-7a5027beffcc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(ProductRetrieveResponse, product, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.contracts.products.with_streaming_response.retrieve(
            id="d84e7f4e-7a70-4fe4-be02-7a5027beffcc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(ProductRetrieveResponse, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_update(self, async_client: AsyncMetronome) -> None:
        product = await async_client.v1.contracts.products.update(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            starting_at=parse_datetime("2020-01-01T00:00:00.000Z"),
        )
        assert_matches_type(ProductUpdateResponse, product, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncMetronome) -> None:
        product = await async_client.v1.contracts.products.update(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            starting_at=parse_datetime("2020-01-01T00:00:00.000Z"),
            billable_metric_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            composite_product_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            composite_tags=["string"],
            exclude_free_usage=True,
            is_refundable=True,
            name="My Updated Product",
            netsuite_internal_item_id="netsuite_internal_item_id",
            netsuite_overage_item_id="netsuite_overage_item_id",
            presentation_group_key=["string"],
            pricing_group_key=["string"],
            quantity_conversion={
                "conversion_factor": 0,
                "operation": "MULTIPLY",
                "name": "name",
            },
            quantity_rounding={
                "decimal_places": 0,
                "rounding_method": "ROUND_UP",
            },
            tags=["string"],
        )
        assert_matches_type(ProductUpdateResponse, product, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.contracts.products.with_raw_response.update(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            starting_at=parse_datetime("2020-01-01T00:00:00.000Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(ProductUpdateResponse, product, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.contracts.products.with_streaming_response.update(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            starting_at=parse_datetime("2020-01-01T00:00:00.000Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(ProductUpdateResponse, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list(self, async_client: AsyncMetronome) -> None:
        product = await async_client.v1.contracts.products.list()
        assert_matches_type(AsyncCursorPage[ProductListResponse], product, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMetronome) -> None:
        product = await async_client.v1.contracts.products.list(
            limit=1,
            next_page="next_page",
            archive_filter="NOT_ARCHIVED",
        )
        assert_matches_type(AsyncCursorPage[ProductListResponse], product, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.contracts.products.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(AsyncCursorPage[ProductListResponse], product, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.contracts.products.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(AsyncCursorPage[ProductListResponse], product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_archive(self, async_client: AsyncMetronome) -> None:
        product = await async_client.v1.contracts.products.archive(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(ProductArchiveResponse, product, path=["response"])

    @parametrize
    async def test_raw_response_archive(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.contracts.products.with_raw_response.archive(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(ProductArchiveResponse, product, path=["response"])

    @parametrize
    async def test_streaming_response_archive(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.contracts.products.with_streaming_response.archive(
            product_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(ProductArchiveResponse, product, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .v2 import (
    V2Resource,
    AsyncV2Resource,
    V2ResourceWithRawResponse,
    AsyncV2ResourceWithRawResponse,
    V2ResourceWithStreamingResponse,
    AsyncV2ResourceWithStreamingResponse,
)
from .contracts import (
    ContractsResource,
    AsyncContractsResource,
    ContractsResourceWithRawResponse,
    AsyncContractsResourceWithRawResponse,
    ContractsResourceWithStreamingResponse,
    AsyncContractsResourceWithStreamingResponse,
)

__all__ = [
    "ContractsResource",
    "AsyncContractsResource",
    "ContractsResourceWithRawResponse",
    "AsyncContractsResourceWithRawResponse",
    "ContractsResourceWithStreamingResponse",
    "AsyncContractsResourceWithStreamingResponse",
    "V2Resource",
    "AsyncV2Resource",
    "V2ResourceWithRawResponse",
    "AsyncV2ResourceWithRawResponse",
    "V2ResourceWithStreamingResponse",
    "AsyncV2ResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .billing_provider_list_params import BillingProviderListParams as BillingProviderListParams
from .billing_provider_create_params import BillingProviderCreateParams as BillingProviderCreateParams
from .billing_provider_list_response import BillingProviderListResponse as BillingProviderListResponse
from .billing_provider_create_response import BillingProviderCreateResponse as BillingProviderCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["InvoiceListBreakdownsParams"]


class InvoiceListBreakdownsParams(TypedDict, total=False):
    customer_id: Required[str]

    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp.

    Breakdowns will only be returned for time windows that end on or before this
    time.
    """

    starting_on: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp.

    Breakdowns will only be returned for time windows that start on or after this
    time.
    """

    credit_type_id: str
    """Only return invoices for the specified credit type"""

    limit: int
    """Max number of results that should be returned.

    For daily breakdowns, the response can return up to 35 days worth of breakdowns.
    For hourly breakdowns, the response can return up to 24 hours. If there are more
    results, a cursor to the next page is returned.
    """

    next_page: str
    """Cursor that indicates where the next page of results should start."""

    skip_zero_qty_line_items: bool
    """If set, all zero quantity line items will be filtered out of the response"""

    sort: Literal["date_asc", "date_desc"]
    """Invoice sort order by issued_at, e.g.

    date_asc or date_desc. Defaults to date_asc.
    """

    status: str
    """Invoice status, e.g. DRAFT or FINALIZED"""

    window_size: Literal["HOUR", "DAY"]
    """The granularity of the breakdowns to return. Defaults to day."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursorPage, AsyncCursorPage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.v1.contracts import (
    product_list_params,
    product_create_params,
    product_update_params,
    product_archive_params,
    product_retrieve_params,
)
from ....types.v1.contracts.product_list_response import ProductListResponse
from ....types.v1.contracts.product_create_response import ProductCreateResponse
from ....types.v1.contracts.product_update_response import ProductUpdateResponse
from ....types.v1.contracts.quantity_rounding_param import QuantityRoundingParam
from ....types.v1.contracts.product_archive_response import ProductArchiveResponse
from ....types.v1.contracts.product_retrieve_response import ProductRetrieveResponse
from ....types.v1.contracts.quantity_conversion_param import QuantityConversionParam

__all__ = ["ProductsResource", "AsyncProductsResource"]


class ProductsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ProductsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return ProductsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProductsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return ProductsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        type: Literal["FIXED", "USAGE", "COMPOSITE", "SUBSCRIPTION", "PROFESSIONAL_SERVICE", "PRO_SERVICE"],
        billable_metric_id: str | Omit = omit,
        composite_product_ids: SequenceNotStr[str] | Omit = omit,
        composite_tags: SequenceNotStr[str] | Omit = omit,
        custom_fields: Dict[str, str] | Omit = omit,
        exclude_free_usage: bool | Omit = omit,
        is_refundable: bool | Omit = omit,
        netsuite_internal_item_id: str | Omit = omit,
        netsuite_overage_item_id: str | Omit = omit,
        presentation_group_key: SequenceNotStr[str] | Omit = omit,
        pricing_group_key: SequenceNotStr[str] | Omit = omit,
        quantity_conversion: Optional[QuantityConversionParam] | Omit = omit,
        quantity_rounding: Optional[QuantityRoundingParam] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductCreateResponse:
        """Create a new product object.

        Products in Metronome represent your company's
        individual product or service offerings. A Product can be thought of as the
        basic unit of a line item on the invoice. This is analogous to SKUs or items in
        an ERP system. Give the product a meaningful name as they will appear on
        customer invoices.

        Args:
          name: displayed on invoices

          billable_metric_id: Required for USAGE products

          composite_product_ids: Required for COMPOSITE products

          composite_tags: Required for COMPOSITE products

          custom_fields: Custom fields to be added eg. { "key1": "value1", "key2": "value2" }

          exclude_free_usage: Beta feature only available for composite products. If true, products with $0
              will not be included when computing composite usage. Defaults to false

          is_refundable: This field's availability is dependent on your client's configuration. Defaults
              to true.

          netsuite_internal_item_id: This field's availability is dependent on your client's configuration.

          netsuite_overage_item_id: This field's availability is dependent on your client's configuration.

          presentation_group_key: For USAGE products only. Groups usage line items on invoices. The superset of
              values in the pricing group key and presentation group key must be set as one
              compound group key on the billable metric.

          pricing_group_key: For USAGE products only. If set, pricing for this product will be determined for
              each pricing_group_key value, as opposed to the product as a whole. The superset
              of values in the pricing group key and presentation group key must be set as one
              compound group key on the billable metric.

          quantity_conversion: Optional. Only valid for USAGE products. If provided, the quantity will be
              converted using the provided conversion factor and operation. For example, if
              the operation is "multiply" and the conversion factor is 100, then the quantity
              will be multiplied by 100. This can be used in cases where data is sent in one
              unit and priced in another. For example, data could be sent in MB and priced in
              GB. In this case, the conversion factor would be 1024 and the operation would be
              "divide".

          quantity_rounding: Optional. Only valid for USAGE products. If provided, the quantity will be
              rounded using the provided rounding method and decimal places. For example, if
              the method is "round up" and the decimal places is 0, then the quantity will be
              rounded up to the nearest integer.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/products/create",
            body=maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "billable_metric_id": billable_metric_id,
                    "composite_product_ids": composite_product_ids,
                    "composite_tags": composite_tags,
                    "custom_fields": custom_fields,
                    "exclude_free_usage": exclude_free_usage,
                    "is_refundable": is_refundable,
                    "netsuite_internal_item_id": netsuite_internal_item_id,
                    "netsuite_overage_item_id": netsuite_overage_item_id,
                    "presentation_group_key": presentation_group_key,
                    "pricing_group_key": pricing_group_key,
                    "quantity_conversion": quantity_conversion,
                    "quantity_rounding": quantity_rounding,
                    "tags": tags,
                },
                product_create_params.ProductCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductCreateResponse,
        )

    def retrieve(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductRetrieveResponse:
        """
        Retrieve a product by its ID, including all metadata and historical changes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/products/get",
            body=maybe_transform({"id": id}, product_retrieve_params.ProductRetrieveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductRetrieveResponse,
        )

    def update(
        self,
        *,
        product_id: str,
        starting_at: Union[str, datetime],
        billable_metric_id: str | Omit = omit,
        composite_product_ids: SequenceNotStr[str] | Omit = omit,
        composite_tags: SequenceNotStr[str] | Omit = omit,
        exclude_free_usage: bool | Omit = omit,
        is_refundable: bool | Omit = omit,
        name: str | Omit = omit,
        netsuite_internal_item_id: str | Omit = omit,
        netsuite_overage_item_id: str | Omit = omit,
        presentation_group_key: SequenceNotStr[str] | Omit = omit,
        pricing_group_key: SequenceNotStr[str] | Omit = omit,
        quantity_conversion: Optional[QuantityConversionParam] | Omit = omit,
        quantity_rounding: Optional[QuantityRoundingParam] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductUpdateResponse:
        """
        Updates a product's configuration while maintaining billing continuity for
        active customers. Use this endpoint to modify product names, metrics, pricing
        rules, and composite settings without disrupting ongoing billing cycles. Changes
        are scheduled using the starting_at timestamp, which must be on an hour
        boundary—set future dates to schedule updates ahead of time, or past dates for
        retroactive changes. Returns the updated product ID upon success.

        ### Usage guidance:

        - Product type cannot be changed after creation. For incorrect product types,
          create a new product and archive the original instead.

        Args:
          product_id: ID of the product to update

          starting_at: Timestamp representing when the update should go into effect. It must be on an
              hour boundary (e.g. 1:00, not 1:30).

          billable_metric_id: Available for USAGE products only. If not provided, defaults to product's
              current billable metric.

          composite_product_ids: Available for COMPOSITE products only. If not provided, defaults to product's
              current composite_product_ids.

          composite_tags: Available for COMPOSITE products only. If not provided, defaults to product's
              current composite_tags.

          exclude_free_usage: Beta feature only available for composite products. If true, products with $0
              will not be included when computing composite usage. Defaults to false

          is_refundable: Defaults to product's current refundability status. This field's availability is
              dependent on your client's configuration.

          name: displayed on invoices. If not provided, defaults to product's current name.

          netsuite_internal_item_id: If not provided, defaults to product's current netsuite_internal_item_id. This
              field's availability is dependent on your client's configuration.

          netsuite_overage_item_id: Available for USAGE and COMPOSITE products only. If not provided, defaults to
              product's current netsuite_overage_item_id. This field's availability is
              dependent on your client's configuration.

          presentation_group_key: For USAGE products only. Groups usage line items on invoices. The superset of
              values in the pricing group key and presentation group key must be set as one
              compound group key on the billable metric.

          pricing_group_key: For USAGE products only. If set, pricing for this product will be determined for
              each pricing_group_key value, as opposed to the product as a whole. The superset
              of values in the pricing group key and presentation group key must be set as one
              compound group key on the billable metric.

          quantity_conversion: Optional. Only valid for USAGE products. If provided, the quantity will be
              converted using the provided conversion factor and operation. For example, if
              the operation is "multiply" and the conversion factor is 100, then the quantity
              will be multiplied by 100. This can be used in cases where data is sent in one
              unit and priced in another. For example, data could be sent in MB and priced in
              GB. In this case, the conversion factor would be 1024 and the operation would be
              "divide".

          quantity_rounding: Optional. Only valid for USAGE products. If provided, the quantity will be
              rounded using the provided rounding method and decimal places. For example, if
              the method is "round up" and the decimal places is 0, then the quantity will be
              rounded up to the nearest integer.

          tags: If not provided, defaults to product's current tags

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/products/update",
            body=maybe_transform(
                {
                    "product_id": product_id,
                    "starting_at": starting_at,
                    "billable_metric_id": billable_metric_id,
                    "composite_product_ids": composite_product_ids,
                    "composite_tags": composite_tags,
                    "exclude_free_usage": exclude_free_usage,
                    "is_refundable": is_refundable,
                    "name": name,
                    "netsuite_internal_item_id": netsuite_internal_item_id,
                    "netsuite_overage_item_id": netsuite_overage_item_id,
                    "presentation_group_key": presentation_group_key,
                    "pricing_group_key": pricing_group_key,
                    "quantity_conversion": quantity_conversion,
                    "quantity_rounding": quantity_rounding,
                    "tags": tags,
                },
                product_update_params.ProductUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductUpdateResponse,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        archive_filter: Literal["ARCHIVED", "NOT_ARCHIVED", "ALL"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[ProductListResponse]:
        """
        Get a paginated list of all products in your organization with their complete
        configuration, version history, and metadata. By default excludes archived
        products unless explicitly requested via the `archive_filter` parameter.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          archive_filter: Filter options for the product list. If not provided, defaults to not archived.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/contract-pricing/products/list",
            page=SyncCursorPage[ProductListResponse],
            body=maybe_transform({"archive_filter": archive_filter}, product_list_params.ProductListParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    product_list_params.ProductListParams,
                ),
            ),
            model=ProductListResponse,
            method="post",
        )

    def archive(
        self,
        *,
        product_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductArchiveResponse:
        """Archive a product.

        Any current rate cards associated with this product will
        continue to function as normal. However, it will no longer be available as an
        option for newly created rates. Once you archive a product, you can still
        retrieve it in the UI and API, but you cannot unarchive it.

        Args:
          product_id: ID of the product to be archived

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/products/archive",
            body=maybe_transform({"product_id": product_id}, product_archive_params.ProductArchiveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductArchiveResponse,
        )


class AsyncProductsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncProductsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncProductsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProductsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncProductsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        type: Literal["FIXED", "USAGE", "COMPOSITE", "SUBSCRIPTION", "PROFESSIONAL_SERVICE", "PRO_SERVICE"],
        billable_metric_id: str | Omit = omit,
        composite_product_ids: SequenceNotStr[str] | Omit = omit,
        composite_tags: SequenceNotStr[str] | Omit = omit,
        custom_fields: Dict[str, str] | Omit = omit,
        exclude_free_usage: bool | Omit = omit,
        is_refundable: bool | Omit = omit,
        netsuite_internal_item_id: str | Omit = omit,
        netsuite_overage_item_id: str | Omit = omit,
        presentation_group_key: SequenceNotStr[str] | Omit = omit,
        pricing_group_key: SequenceNotStr[str] | Omit = omit,
        quantity_conversion: Optional[QuantityConversionParam] | Omit = omit,
        quantity_rounding: Optional[QuantityRoundingParam] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductCreateResponse:
        """Create a new product object.

        Products in Metronome represent your company's
        individual product or service offerings. A Product can be thought of as the
        basic unit of a line item on the invoice. This is analogous to SKUs or items in
        an ERP system. Give the product a meaningful name as they will appear on
        customer invoices.

        Args:
          name: displayed on invoices

          billable_metric_id: Required for USAGE products

          composite_product_ids: Required for COMPOSITE products

          composite_tags: Required for COMPOSITE products

          custom_fields: Custom fields to be added eg. { "key1": "value1", "key2": "value2" }

          exclude_free_usage: Beta feature only available for composite products. If true, products with $0
              will not be included when computing composite usage. Defaults to false

          is_refundable: This field's availability is dependent on your client's configuration. Defaults
              to true.

          netsuite_internal_item_id: This field's availability is dependent on your client's configuration.

          netsuite_overage_item_id: This field's availability is dependent on your client's configuration.

          presentation_group_key: For USAGE products only. Groups usage line items on invoices. The superset of
              values in the pricing group key and presentation group key must be set as one
              compound group key on the billable metric.

          pricing_group_key: For USAGE products only. If set, pricing for this product will be determined for
              each pricing_group_key value, as opposed to the product as a whole. The superset
              of values in the pricing group key and presentation group key must be set as one
              compound group key on the billable metric.

          quantity_conversion: Optional. Only valid for USAGE products. If provided, the quantity will be
              converted using the provided conversion factor and operation. For example, if
              the operation is "multiply" and the conversion factor is 100, then the quantity
              will be multiplied by 100. This can be used in cases where data is sent in one
              unit and priced in another. For example, data could be sent in MB and priced in
              GB. In this case, the conversion factor would be 1024 and the operation would be
              "divide".

          quantity_rounding: Optional. Only valid for USAGE products. If provided, the quantity will be
              rounded using the provided rounding method and decimal places. For example, if
              the method is "round up" and the decimal places is 0, then the quantity will be
              rounded up to the nearest integer.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/products/create",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "billable_metric_id": billable_metric_id,
                    "composite_product_ids": composite_product_ids,
                    "composite_tags": composite_tags,
                    "custom_fields": custom_fields,
                    "exclude_free_usage": exclude_free_usage,
                    "is_refundable": is_refundable,
                    "netsuite_internal_item_id": netsuite_internal_item_id,
                    "netsuite_overage_item_id": netsuite_overage_item_id,
                    "presentation_group_key": presentation_group_key,
                    "pricing_group_key": pricing_group_key,
                    "quantity_conversion": quantity_conversion,
                    "quantity_rounding": quantity_rounding,
                    "tags": tags,
                },
                product_create_params.ProductCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductCreateResponse,
        )

    async def retrieve(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductRetrieveResponse:
        """
        Retrieve a product by its ID, including all metadata and historical changes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/products/get",
            body=await async_maybe_transform({"id": id}, product_retrieve_params.ProductRetrieveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductRetrieveResponse,
        )

    async def update(
        self,
        *,
        product_id: str,
        starting_at: Union[str, datetime],
        billable_metric_id: str | Omit = omit,
        composite_product_ids: SequenceNotStr[str] | Omit = omit,
        composite_tags: SequenceNotStr[str] | Omit = omit,
        exclude_free_usage: bool | Omit = omit,
        is_refundable: bool | Omit = omit,
        name: str | Omit = omit,
        netsuite_internal_item_id: str | Omit = omit,
        netsuite_overage_item_id: str | Omit = omit,
        presentation_group_key: SequenceNotStr[str] | Omit = omit,
        pricing_group_key: SequenceNotStr[str] | Omit = omit,
        quantity_conversion: Optional[QuantityConversionParam] | Omit = omit,
        quantity_rounding: Optional[QuantityRoundingParam] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductUpdateResponse:
        """
        Updates a product's configuration while maintaining billing continuity for
        active customers. Use this endpoint to modify product names, metrics, pricing
        rules, and composite settings without disrupting ongoing billing cycles. Changes
        are scheduled using the starting_at timestamp, which must be on an hour
        boundary—set future dates to schedule updates ahead of time, or past dates for
        retroactive changes. Returns the updated product ID upon success.

        ### Usage guidance:

        - Product type cannot be changed after creation. For incorrect product types,
          create a new product and archive the original instead.

        Args:
          product_id: ID of the product to update

          starting_at: Timestamp representing when the update should go into effect. It must be on an
              hour boundary (e.g. 1:00, not 1:30).

          billable_metric_id: Available for USAGE products only. If not provided, defaults to product's
              current billable metric.

          composite_product_ids: Available for COMPOSITE products only. If not provided, defaults to product's
              current composite_product_ids.

          composite_tags: Available for COMPOSITE products only. If not provided, defaults to product's
              current composite_tags.

          exclude_free_usage: Beta feature only available for composite products. If true, products with $0
              will not be included when computing composite usage. Defaults to false

          is_refundable: Defaults to product's current refundability status. This field's availability is
              dependent on your client's configuration.

          name: displayed on invoices. If not provided, defaults to product's current name.

          netsuite_internal_item_id: If not provided, defaults to product's current netsuite_internal_item_id. This
              field's availability is dependent on your client's configuration.

          netsuite_overage_item_id: Available for USAGE and COMPOSITE products only. If not provided, defaults to
              product's current netsuite_overage_item_id. This field's availability is
              dependent on your client's configuration.

          presentation_group_key: For USAGE products only. Groups usage line items on invoices. The superset of
              values in the pricing group key and presentation group key must be set as one
              compound group key on the billable metric.

          pricing_group_key: For USAGE products only. If set, pricing for this product will be determined for
              each pricing_group_key value, as opposed to the product as a whole. The superset
              of values in the pricing group key and presentation group key must be set as one
              compound group key on the billable metric.

          quantity_conversion: Optional. Only valid for USAGE products. If provided, the quantity will be
              converted using the provided conversion factor and operation. For example, if
              the operation is "multiply" and the conversion factor is 100, then the quantity
              will be multiplied by 100. This can be used in cases where data is sent in one
              unit and priced in another. For example, data could be sent in MB and priced in
              GB. In this case, the conversion factor would be 1024 and the operation would be
              "divide".

          quantity_rounding: Optional. Only valid for USAGE products. If provided, the quantity will be
              rounded using the provided rounding method and decimal places. For example, if
              the method is "round up" and the decimal places is 0, then the quantity will be
              rounded up to the nearest integer.

          tags: If not provided, defaults to product's current tags

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/products/update",
            body=await async_maybe_transform(
                {
                    "product_id": product_id,
                    "starting_at": starting_at,
                    "billable_metric_id": billable_metric_id,
                    "composite_product_ids": composite_product_ids,
                    "composite_tags": composite_tags,
                    "exclude_free_usage": exclude_free_usage,
                    "is_refundable": is_refundable,
                    "name": name,
                    "netsuite_internal_item_id": netsuite_internal_item_id,
                    "netsuite_overage_item_id": netsuite_overage_item_id,
                    "presentation_group_key": presentation_group_key,
                    "pricing_group_key": pricing_group_key,
                    "quantity_conversion": quantity_conversion,
                    "quantity_rounding": quantity_rounding,
                    "tags": tags,
                },
                product_update_params.ProductUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductUpdateResponse,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        archive_filter: Literal["ARCHIVED", "NOT_ARCHIVED", "ALL"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ProductListResponse, AsyncCursorPage[ProductListResponse]]:
        """
        Get a paginated list of all products in your organization with their complete
        configuration, version history, and metadata. By default excludes archived
        products unless explicitly requested via the `archive_filter` parameter.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          archive_filter: Filter options for the product list. If not provided, defaults to not archived.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/contract-pricing/products/list",
            page=AsyncCursorPage[ProductListResponse],
            body=maybe_transform({"archive_filter": archive_filter}, product_list_params.ProductListParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    product_list_params.ProductListParams,
                ),
            ),
            model=ProductListResponse,
            method="post",
        )

    async def archive(
        self,
        *,
        product_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductArchiveResponse:
        """Archive a product.

        Any current rate cards associated with this product will
        continue to function as normal. However, it will no longer be available as an
        option for newly created rates. Once you archive a product, you can still
        retrieve it in the UI and API, but you cannot unarchive it.

        Args:
          product_id: ID of the product to be archived

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/products/archive",
            body=await async_maybe_transform({"product_id": product_id}, product_archive_params.ProductArchiveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductArchiveResponse,
        )


class ProductsResourceWithRawResponse:
    def __init__(self, products: ProductsResource) -> None:
        self._products = products

        self.create = to_raw_response_wrapper(
            products.create,
        )
        self.retrieve = to_raw_response_wrapper(
            products.retrieve,
        )
        self.update = to_raw_response_wrapper(
            products.update,
        )
        self.list = to_raw_response_wrapper(
            products.list,
        )
        self.archive = to_raw_response_wrapper(
            products.archive,
        )


class AsyncProductsResourceWithRawResponse:
    def __init__(self, products: AsyncProductsResource) -> None:
        self._products = products

        self.create = async_to_raw_response_wrapper(
            products.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            products.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            products.update,
        )
        self.list = async_to_raw_response_wrapper(
            products.list,
        )
        self.archive = async_to_raw_response_wrapper(
            products.archive,
        )


class ProductsResourceWithStreamingResponse:
    def __init__(self, products: ProductsResource) -> None:
        self._products = products

        self.create = to_streamed_response_wrapper(
            products.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            products.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            products.update,
        )
        self.list = to_streamed_response_wrapper(
            products.list,
        )
        self.archive = to_streamed_response_wrapper(
            products.archive,
        )


class AsyncProductsResourceWithStreamingResponse:
    def __init__(self, products: AsyncProductsResource) -> None:
        self._products = products

        self.create = async_to_streamed_response_wrapper(
            products.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            products.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            products.update,
        )
        self.list = async_to_streamed_response_wrapper(
            products.list,
        )
        self.archive = async_to_streamed_response_wrapper(
            products.archive,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .plans import (
    PlansResource,
    AsyncPlansResource,
    PlansResourceWithRawResponse,
    AsyncPlansResourceWithRawResponse,
    PlansResourceWithStreamingResponse,
    AsyncPlansResourceWithStreamingResponse,
)
from .alerts import (
    AlertsResource,
    AsyncAlertsResource,
    AlertsResourceWithRawResponse,
    AsyncAlertsResourceWithRawResponse,
    AlertsResourceWithStreamingResponse,
    AsyncAlertsResourceWithStreamingResponse,
)
from .commits import (
    CommitsResource,
    AsyncCommitsResource,
    CommitsResourceWithRawResponse,
    AsyncCommitsResourceWithRawResponse,
    CommitsResourceWithStreamingResponse,
    AsyncCommitsResourceWithStreamingResponse,
)
from .credits import (
    CreditsResource,
    AsyncCreditsResource,
    CreditsResourceWithRawResponse,
    AsyncCreditsResourceWithRawResponse,
    CreditsResourceWithStreamingResponse,
    AsyncCreditsResourceWithStreamingResponse,
)
from .invoices import (
    InvoicesResource,
    AsyncInvoicesResource,
    InvoicesResourceWithRawResponse,
    AsyncInvoicesResourceWithRawResponse,
    InvoicesResourceWithStreamingResponse,
    AsyncInvoicesResourceWithStreamingResponse,
)
from .customers import (
    CustomersResource,
    AsyncCustomersResource,
    CustomersResourceWithRawResponse,
    AsyncCustomersResourceWithRawResponse,
    CustomersResourceWithStreamingResponse,
    AsyncCustomersResourceWithStreamingResponse,
)
from .billing_config import (
    BillingConfigResource,
    AsyncBillingConfigResource,
    BillingConfigResourceWithRawResponse,
    AsyncBillingConfigResourceWithRawResponse,
    BillingConfigResourceWithStreamingResponse,
    AsyncBillingConfigResourceWithStreamingResponse,
)
from .named_schedules import (
    NamedSchedulesResource,
    AsyncNamedSchedulesResource,
    NamedSchedulesResourceWithRawResponse,
    AsyncNamedSchedulesResourceWithRawResponse,
    NamedSchedulesResourceWithStreamingResponse,
    AsyncNamedSchedulesResourceWithStreamingResponse,
)

__all__ = [
    "AlertsResource",
    "AsyncAlertsResource",
    "AlertsResourceWithRawResponse",
    "AsyncAlertsResourceWithRawResponse",
    "AlertsResourceWithStreamingResponse",
    "AsyncAlertsResourceWithStreamingResponse",
    "PlansResource",
    "AsyncPlansResource",
    "PlansResourceWithRawResponse",
    "AsyncPlansResourceWithRawResponse",
    "PlansResourceWithStreamingResponse",
    "AsyncPlansResourceWithStreamingResponse",
    "InvoicesResource",
    "AsyncInvoicesResource",
    "InvoicesResourceWithRawResponse",
    "AsyncInvoicesResourceWithRawResponse",
    "InvoicesResourceWithStreamingResponse",
    "AsyncInvoicesResourceWithStreamingResponse",
    "BillingConfigResource",
    "AsyncBillingConfigResource",
    "BillingConfigResourceWithRawResponse",
    "AsyncBillingConfigResourceWithRawResponse",
    "BillingConfigResourceWithStreamingResponse",
    "AsyncBillingConfigResourceWithStreamingResponse",
    "CommitsResource",
    "AsyncCommitsResource",
    "CommitsResourceWithRawResponse",
    "AsyncCommitsResourceWithRawResponse",
    "CommitsResourceWithStreamingResponse",
    "AsyncCommitsResourceWithStreamingResponse",
    "CreditsResource",
    "AsyncCreditsResource",
    "CreditsResourceWithRawResponse",
    "AsyncCreditsResourceWithRawResponse",
    "CreditsResourceWithStreamingResponse",
    "AsyncCreditsResourceWithStreamingResponse",
    "NamedSchedulesResource",
    "AsyncNamedSchedulesResource",
    "NamedSchedulesResourceWithRawResponse",
    "AsyncNamedSchedulesResourceWithRawResponse",
    "NamedSchedulesResourceWithStreamingResponse",
    "AsyncNamedSchedulesResourceWithStreamingResponse",
    "CustomersResource",
    "AsyncCustomersResource",
    "CustomersResourceWithRawResponse",
    "AsyncCustomersResourceWithRawResponse",
    "CustomersResourceWithStreamingResponse",
    "AsyncCustomersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["CustomerSetBillingConfigurationsParams", "Data"]


class CustomerSetBillingConfigurationsParams(TypedDict, total=False):
    data: Required[Iterable[Data]]


class Data(TypedDict, total=False):
    billing_provider: Required[
        Literal[
            "aws_marketplace",
            "stripe",
            "netsuite",
            "custom",
            "azure_marketplace",
            "quickbooks_online",
            "workday",
            "gcp_marketplace",
            "metronome",
        ]
    ]
    """The billing provider set for this configuration."""

    customer_id: Required[str]

    configuration: Dict[str, object]
    """Configuration for the billing provider.

    The structure of this object is specific to the billing provider and delivery
    method combination. Defaults to an empty object, however, for most billing
    provider + delivery method combinations, it will not be a valid configuration.
    For AWS marketplace configurations, the aws_is_subscription_product flag can be
    used to indicate a product with usage-based pricing. More information can be
    found
    [here](https://docs.metronome.com/invoice-customers/solutions/marketplaces/invoice-aws/#provision-aws-marketplace-customers-in-metronome).
    """

    delivery_method: Literal["direct_to_billing_provider", "aws_sqs", "tackle", "aws_sns"]
    """The method to use for delivering invoices to this customer.

    If not provided, the `delivery_method_id` must be provided.
    """

    delivery_method_id: str
    """ID of the delivery method to use for this customer.

    If not provided, the `delivery_method` must be provided.
    """

    tax_provider: Literal["anrok", "avalara", "stripe"]
    """
    Specifies which tax provider Metronome should use for tax calculation when
    billing through Stripe. This is only supported for Stripe billing provider
    configurations with auto_charge_payment_intent or manual_charge_payment_intent
    collection methods.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AuditLogListResponse", "Request", "Actor"]


class Request(BaseModel):
    id: str

    ip: Optional[str] = None

    user_agent: Optional[str] = None


class Actor(BaseModel):
    id: str

    name: str

    email: Optional[str] = None


class AuditLogListResponse(BaseModel):
    id: str

    request: Request

    timestamp: datetime

    action: Optional[str] = None

    actor: Optional[Actor] = None

    description: Optional[str] = None

    resource_id: Optional[str] = None

    resource_type: Optional[str] = None

    status: Optional[Literal["success", "failure", "pending"]] = None
